<?php

namespace Drupal\flexible_descriptions;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\flexible_descriptions\Entity\FlexibleDescriptions;
use Drupal\flexible_descriptions\Form\FlexibleDescriptionsSettingsForm;

/**
 * Provides a helper service with useful methods for flexible descriptions.
 */
class FlexibleDescriptionsHelper implements FlexibleDescriptionsHelperInterface {

  /**
   * The Entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * Constructs FlexibleDescriptionsHelper object.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    ConfigFactoryInterface $config_factory,
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->configFactory = $config_factory;
  }

  /**
   * Returns an array of supported entity types.
   */
  public function getSupportedEntityTypes(): array {
    $entity_types_info = $this->configFactory->get(FlexibleDescriptionsSettingsForm::CONFIG_NAME)->get('entity_types');
    $entity_types = [];
    foreach ($entity_types_info as $entity_type_id => $entity_type_info) {
      if ($entity_type_info['is_supported']) {
        $entity_types[] = $this->entityTypeManager->getDefinition($entity_type_id);
      }
    }

    return !empty($entity_types) ? $entity_types : [];
  }

  /**
   * {@inheritdoc}
   */
  public function generateDescriptionIdentifier(
    string $entity_type_id,
    string $bundle_id,
    string $field_name
  ): string {
    return implode('|', [
      $entity_type_id,
      $bundle_id,
      $field_name,
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getExistingDescription(string $description_identifier): array|int {
    return $this->entityTypeManager->getStorage('flexible_description')
      ->getQuery()
      ->accessCheck(FALSE)
      ->condition('description_identifier', $description_identifier)
      ->execute();
  }

  /**
   * {@inheritdoc}
   */
  public function getFieldWidgetDescriptionParents(array $field_widget_complete_form): ?array {
    foreach ($field_widget_complete_form as $key => $value) {
      if ($key === '#description') {
        return [$key];
      }
      if (is_array($value)) {
        $nested_path = $this->getFieldWidgetDescriptionParents($value);
        if ($nested_path !== NULL) {
          array_unshift($nested_path, $key);
          return $nested_path;
        }
      }
    }
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getFlexibleDescriptionText(string $description_identifier) : string {
    $description_text = '';
    $flexible_description_storage = $this->entityTypeManager->getStorage('flexible_description');
    $flexible_description_id = $flexible_description_storage
      ->getQuery()
      ->accessCheck(FALSE)
      ->condition('description_identifier', $description_identifier)
      ->execute();
    if (!empty($flexible_description_id) && is_array($flexible_description_id)) {
      $flexible_description = $flexible_description_storage->load(reset($flexible_description_id));
      if ($flexible_description instanceof FlexibleDescriptions && !$flexible_description->get('description_text')->isEmpty()) {
        $description_text = $flexible_description->get('description_text')->getString();
      }
    }

    return $description_text;
  }

}
